@@ -1,10 +1,9 @@
 package Net::DNS::RR;
 
-#
-# $Id: RR.pm 1475 2016-04-15 09:46:40Z willem $
-#
-use vars qw($VERSION);
-$VERSION = (qw$LastChangedRevision: 1475 $)[1];
+use strict;
+use warnings;
+
+our $VERSION = (qw$Id: RR.pm 1812 2020-10-07 18:09:53Z willem $)[2];
 
 
 =head1 NAME
@@ -15,12 +14,12 @@ Net::DNS::RR - DNS resource record base class
 
     use Net::DNS;
 
-    $rr = new Net::DNS::RR('example.com IN A 192.0.2.99');
+    $rr = Net::DNS::RR->new('example.com IN AAAA 2001:DB8::1');
 
-    $rr = new Net::DNS::RR(
-	    name    => 'example.com',
-	    type    => 'A',
-	    address => '192.0.2.99'
+    $rr = Net::DNS::RR->new(
+	    owner   => 'example.com',
+	    type    => 'AAAA',
+	    address => '2001:DB8::1'
 	    );
 
 
@@ -32,12 +31,12 @@ See also the manual pages for each specific RR type.
 =cut
 
 
-use strict;
 use integer;
 use Carp;
 
-use Net::DNS::Parameters;
-use Net::DNS::Domain;
+use constant LIB => grep { $_ ne '.' } grep { !ref($_) } @INC;
+
+use Net::DNS::Parameters qw(%classbyname :class :type);
 use Net::DNS::DomainName;
 
 
@@ -56,8 +55,8 @@ sub new {
 		scalar @_ > 2 ? &_new_hash : &_new_string;
 	} || do {
 		my $class = shift || __PACKAGE__;
-		my @param = map defined($_) ? split : 'undef', @_;
-		my $stmnt = substr "new $class( @param )", 0, 80;
+		my @param = map { defined($_) ? split /\s+/ : 'undef' } @_;
+		my $stmnt = substr "$class->new( @param )", 0, 80;
 		croak "${@}in $stmnt\n";
 	};
 }
@@ -65,29 +64,28 @@ sub new {
 
 =head2 new (from string)
 
-    $a	   = new Net::DNS::RR('host.example.com. 86400 A 192.0.2.1');
-    $mx	   = new Net::DNS::RR('example.com. 7200 MX 10 mailhost.example.com.');
-    $cname = new Net::DNS::RR('www.example.com 300 IN CNAME host.example.com');
-    $txt   = new Net::DNS::RR('txt.example.com 3600 HS TXT "text data"');
+    $aaaa  = Net::DNS::RR->new('host.example.com. 86400 AAAA 2001:DB8::1');
+    $mx	   = Net::DNS::RR->new('example.com. 7200 MX 10 mailhost.example.com.');
+    $cname = Net::DNS::RR->new('www.example.com 300 IN CNAME host.example.com');
+    $txt   = Net::DNS::RR->new('txt.example.com 3600 HS TXT "text data"');
 
-Returns an RR object of the appropriate type, or a C<Net::DNS::RR>
-object if the type is not implemented.	The attribute values are
-extracted from the string passed by the user.  The syntax of the
-argument string follows the RFC1035 specification for zone files,
-and is compatible with the result returned by the string method.
+Returns an object of the appropriate RR type, or a L<Net::DNS::RR> object
+if the type is not implemented. The attribute values are extracted from the
+string passed by the user. The syntax of the argument string follows the
+RFC1035 specification for zone files, and is compatible with the result
+returned by the string method.
 
-The name and RR type are required; all other information is optional.
-If omitted, the TTL defaults to 0 and the RR class defaults to IN.
+The owner and RR type are required; all other information is optional.
 Omitting the optional fields is useful for creating the empty RDATA
-sections required for certain dynamic update operations.  See the
-C<Net::DNS::Update> manual page for additional examples.
+sections required for certain dynamic update operations.
+See the L<Net::DNS::Update> manual page for additional examples.
 
 All names are interpreted as fully qualified domain names.
 The trailing dot (.) is optional.
 
 =cut
 
-my $PARSE_REGEX = q/("[^"]*")|;[^\n]*|[ \t\n\r\f()]/;
+my $PARSE_REGEX = q/("[^"]*")|;[^\n]*|[ \t\n\r\f()]/;		# NB: *not* \s (matches Unicode white space)
 
 sub _new_string {
 	my $base;
@@ -102,27 +100,28 @@ sub _new_string {
 	s/\\\(/\\040/g;						# disguise escaped bracket
 	s/\\\)/\\041/g;						# disguise escaped bracket
 	s/\\;/\\059/g;						# disguise escaped semicolon
-	my ( $name, @token ) = grep defined && length, split /$PARSE_REGEX/o;
+	my ( $owner, @token ) = grep { defined && length } split /$PARSE_REGEX/o;
 
-	my ( $t1, $t2, $t3 ) = @token;
-	croak 'unable to parse RR string' unless defined $t1;
+	croak 'unable to parse RR string' unless scalar @token;
+	my $t1 = uc $token[0];
+	my $t2 = $token[1];
 
 	my ( $ttl, $class );
-	unless ( defined $t2 ) {				# <name> <type>
-		@token = ('ANY') if $classbyname{$t1};		# <name> <class>
-	} elsif ( $classbyname{$t1} || $t1 =~ /^CLASS\d/ ) {
-		$class = shift @token;				# <name> <class> [<ttl>] <type>
-		$ttl = shift @token if $t2 =~ /^\d/;
+	if ( not defined $t2 ) {				# <owner> <type>
+		@token = ('ANY') if $classbyname{$t1};		# <owner> <class>
 	} elsif ( $t1 =~ /^\d/ ) {
-		$ttl = shift @token;				# <name> <ttl> [<class>] <type>
-		$class = shift @token if $classbyname{$t2} || $t2 =~ /^CLASS\d/;
+		$ttl   = shift @token;				# <owner> <ttl> [<class>] <type>
+		$class = shift @token if $classbyname{uc $t2} || $t2 =~ /^CLASS\d/i;
+	} elsif ( $classbyname{$t1} || $t1 =~ /^CLASS\d/ ) {
+		$class = shift @token;				# <owner> <class> [<ttl>] <type>
+		$ttl   = shift @token if $t2 =~ /^\d/;
 	}
 
 	my $type      = shift(@token);
 	my $populated = scalar @token;
 
 	my $self = $base->_subclass( $type, $populated );	# create RR object
-	$self->name($name);
+	$self->owner($owner);
 	$self->class($class) if defined $class;			# specify CLASS
 	$self->ttl($ttl)     if defined $ttl;			# specify TTL
 
@@ -130,70 +129,72 @@ sub _new_string {
 
 	if ( $#token && $token[0] =~ /^[\\]?#$/ ) {
 		shift @token;					# RFC3597 hexadecimal format
-		my $count = shift(@token) || 0;
-		my $rdata = pack 'H*', join '', @token;
-		my $rdlen = $self->{rdlength} = length $rdata;
-		croak 'length and hexadecimal data inconsistent' unless $rdlen == $count;
-		$self->_decode_rdata( \$rdata, 0 ) if $rdlen;	# unpack RDATA
+		my $rdlen = shift(@token) || 0;
+		my $rdata = pack 'H*', join( '', @token );
+		croak 'length and hexadecimal data inconsistent' unless $rdlen == length $rdata;
+		$self->rdata($rdata);				# unpack RDATA
 		return $self;
 	}
 
 	$self->_parse_rdata(@token);				# parse arguments
+	$self->_post_parse();
 	return $self;
 }
 
 
 =head2 new (from hash)
 
-    $rr = new Net::DNS::RR(%hash);
+    $rr = Net::DNS::RR->new(%hash);
 
-    $rr = new Net::DNS::RR(
-	    name    => 'host.example.com',
+    $rr = Net::DNS::RR->new(
+	    owner   => 'host.example.com',
 	    ttl	    => 86400,
 	    class   => 'IN',
-	    type    => 'A',
-	    address => '192.0.2.1'
+	    type    => 'AAAA',
+	    address => '2001:DB8::1'
 	    );
  
-    $rr = new Net::DNS::RR(
-	    name    => 'txt.example.com',
+    $rr = Net::DNS::RR->new(
+	    owner   => 'txt.example.com',
 	    type    => 'TXT',
 	    txtdata => [ 'one', 'two' ]
 	    );
 
-Returns an RR object of the appropriate type, or a C<Net::DNS::RR>
-object if the type is not implemented.	See the manual pages for
-each RR type to see what fields the type requires.
+Returns an object of the appropriate RR type, or a L<Net::DNS::RR> object
+if the type is not implemented. Consult the relevant manual pages for the
+usage of type specific attributes.
 
-The C<name> and C<type> fields are required; all others are optional.
-If omitted, C<ttl> defaults to 0 and C<class> defaults to IN.
-Omitting the optional fields is useful for creating the empty RDATA
+The owner and RR type are required; all other information is optional.
+Omitting optional attributes is useful for creating the empty RDATA
 sections required for certain dynamic update operations.
 
 =cut
 
+my @core = qw(owner name type class ttl rdlength);
+
 sub _new_hash {
-	my ( $base, %argument ) = @_;
+	my $base = shift;
 
-	my %attribute = ( name => '.', type => 'ANY' );
-	while ( my ( $key, $value ) = each %argument ) {
-		$attribute{lc $key} = $value;
+	my %attribute = ( owner => '.', type => 'NULL' );
+	while ( my $key = shift ) {
+		$attribute{lc $key} = shift;
 	}
 
-	my ( $name, $type, $class, $ttl ) = @attribute{qw(name type class ttl)};
-	delete @attribute{qw(name class type ttl rdlength)};	# strip non-RDATA fields
+	my ( $owner, $name, $type, $class, $ttl ) = delete @attribute{@core};
 
-	my $populated = scalar %attribute;			# RDATA specified
+	my $self = $base->_subclass( $type, scalar(%attribute) );
+	$self->owner( $name ? $name : $owner );
+	$self->class($class) if defined $class;			# optional CLASS
+	$self->ttl($ttl)     if defined $ttl;			# optional TTL
 
-	my $self = $base->_subclass( $type, $populated );	# RR with defaults (if appropriate)
-	$self->name($name);
-	$self->class($class) if defined $class;			# specify CLASS
-	$self->ttl($ttl)     if defined $ttl;			# specify TTL
-
-	while ( my ( $attribute, $value ) = each %attribute ) {
-		$self->$attribute( ref($value) eq 'ARRAY' ? @$value : $value );
-	}
+	eval {
+		while ( my ( $attribute, $value ) = each %attribute ) {
+			$self->$attribute( ref($value) eq 'ARRAY' ? @$value : $value );
+		}
+	};
+	die ref($self) eq __PACKAGE__ ? "type $type not implemented" : () if $@;
 
+	$self->_post_parse();
 	return $self;
 }
 
@@ -231,8 +232,7 @@ sub decode {
 
 	my $index = $fixed + RRFIXEDSZ;
 	die 'corrupt wire-format data' if length $$data < $index;
-	my $type = unpack "\@$fixed n", $$data;
-	my $self = $base->_subclass( typebyval($type) );
+	my $self = $base->_subclass( unpack "\@$fixed n", $$data );
 	$self->{owner} = $owner;
 	@{$self}{qw(class ttl rdlength)} = unpack "\@$fixed x2 n N n", $$data;
 
@@ -267,11 +267,9 @@ sub encode {
 	my ( $offset, @opaque ) = scalar(@_) ? @_ : ( 0x4000, {} );
 
 	my $owner = $self->{owner}->encode( $offset, @opaque );
-	my $type  = $self->{type};
-	my $class = $self->{class} || 1;
-	my $index = $offset + length($owner) + RRFIXEDSZ;
-	my $rdata = eval { $self->_encode_rdata( $index, @opaque ); } || '';
-	return pack 'a* n2 N n a*', $owner, $type, $class, $self->ttl, length $rdata, $rdata;
+	my ( $type, $class, $ttl ) = @{$self}{qw(type class ttl)};
+	my $rdata = $self->_empty ? '' : $self->_encode_rdata( $offset + length($owner) + RRFIXEDSZ, @opaque );
+	return pack 'a* n2 N n a*', $owner, $type, $class || 1, $ttl || 0, length $rdata, $rdata;
 }
 
 
@@ -292,11 +290,9 @@ sub canonical {
 	my $self = shift;
 
 	my $owner = $self->{owner}->canonical;
-	my $type  = $self->{type};
-	my $class = $self->{class} || 1;
-	my $index = RRFIXEDSZ + length $owner;
-	my $rdata = eval { $self->_encode_rdata($index); } || '';
-	pack 'a* n2 N n a*', $owner, $type, $class, $self->ttl, length $rdata, $rdata;
+	my ( $type, $class, $ttl ) = @{$self}{qw(type class ttl)};
+	my $rdata = $self->_empty ? '' : $self->_encode_rdata( length($owner) + RRFIXEDSZ );
+	return pack 'a* n2 N n a*', $owner, $type, $class || 1, $ttl || 0, length $rdata, $rdata;
 }
 
 
@@ -304,13 +300,14 @@ sub canonical {
 
     $rr->print;
 
-Prints the record to the standard output.  Calls the string method
-to get the formatted RR representation.
+Prints the resource record to the currently selected output filehandle.
+Calls the string method to get the formatted RR representation.
 
 =cut
 
 sub print {
 	print shift->string, "\n";
+	return;
 }
 
 
@@ -318,39 +315,115 @@ sub print {
 
     print $rr->string, "\n";
 
-Returns a string representation of the RR using the zone file format
-described in RFC1035.  All domain names are fully qualified with
-trailing dot.  This differs from RR attribute methods, which omit
-the trailing dot.
+Returns a string representation of the RR using the master file format
+mandated by RFC1035.
+All domain names are fully qualified with trailing dot.
+This differs from RR attribute methods, which omit the trailing dot.
 
 =cut
 
 sub string {
 	my $self = shift;
 
-	my @ttl = grep defined, ( $self->{ttl} );
-	my $core = join "\t", $self->{owner}->string, @ttl, $self->class, $self->type;
+	my $name = $self->{owner}->string;
+	my @ttl	 = grep {defined} $self->{ttl};
+	my @core = ( $name, @ttl, $self->class, $self->type );
 
-	my $rdata = $self->rdstring( 14 + length $core );
-	return join "\t", $core, length $rdata ? $rdata : '; no data';
+	my $empty = $self->_empty;
+	my @rdata = $empty ? () : eval { $self->_format_rdata };
+	carp $@ if $@;
+
+	my $tab = length($name) < 72 ? "\t" : ' ';
+	$self->_annotation('no data') if $empty;
+
+	my @line = _wrap( join( $tab, @core, '(' ), @rdata, ')' );
+
+	my $last = pop(@line);					# last or only line
+	$last = join $tab, @core, "@rdata" unless scalar(@line);
+
+	return join "\n\t", @line, _wrap( $last, map {"; $_"} $self->_annotation );
+}
+
+
+=head2 plain
+
+    $plain = $rr->plain;
+
+Returns a simplified single-line representation of the RR.
+This facilitates interaction with programs like nsupdate
+which have rudimentary parsers.
+
+=cut
+
+sub plain {
+	my $string = join ' ', shift->token;
+	$string =~ s/\\034/\\"/g;				# unescape "
+	$string =~ s/\\092/\\\\/g;				# unescape escape
+	return $string;
+}
+
+
+=head2 token
+
+    @token = $rr->token;
+
+Returns a token list representation of the RR zone file string.
+
+=cut
+
+sub token {
+	my $self = shift;
+
+	my @ttl	 = grep {defined} $self->{ttl};
+	my @core = ( $self->{owner}->string, @ttl, $self->class, $self->type );
+
+	my @rdata = $self->_empty ? () : eval { $self->_format_rdata };
+
+	# parse into quoted strings, contiguous non-whitespace and (discarded) comments
+	my @parse = map { split( /$PARSE_REGEX/o, $_ ) } @rdata;
+	return ( @core, grep { defined && length } @parse );
+}
+
+
+=head2 generic
+
+    $generic = $rr->generic;
+
+Returns the generic RR representation defined in RFC3597. This facilitates
+creation of zone files containing RRs unrecognised by outdated nameservers
+and provisioning software.
+
+=cut
+
+sub generic {
+	my $self = shift;
+
+	my @ttl	  = grep {defined} $self->{ttl};
+	my @class = map	 {"CLASS$_"} grep {defined} $self->{class};
+	my @core  = ( $self->{owner}->string, @ttl, @class, "TYPE$self->{type}" );
+	my $data  = $self->rdata;
+	my @data  = ( '\\#', length($data), split /(\S{32})/, unpack 'H*', $data );
+	my @line  = _wrap( "@core (", @data, ')' );
+	return join "\n\t", @line if scalar(@line) > 1;
+	return join ' ', @core, @data;
 }
 
 
 =head2 owner name
 
-    $owner = $rr->name;
+    $name = $rr->owner;
 
 Returns the owner name of the record.
 
 =cut
 
-sub name {
+sub owner {
 	my $self = shift;
-	$self->{owner} = new Net::DNS::DomainName1035(shift) if scalar @_;
-	$self->{owner}->name if defined wantarray;
+	$self->{owner} = Net::DNS::DomainName1035->new(shift) if scalar @_;
+	return defined wantarray ? $self->{owner}->name : undef;
 }
 
-sub owner { &name; }			## compatibility with RFC1034
+sub name { return &owner; }		## historical
 
 
 =head2 type
@@ -364,7 +437,7 @@ Returns the record type.
 sub type {
 	my $self = shift;
 	croak 'not possible to change RR->type' if scalar @_;
-	typebyval( $self->{type} );
+	return typebyval( $self->{type} );
 }
 
 
@@ -378,8 +451,8 @@ Resource record class.
 
 sub class {
 	my $self = shift;
-	$self->{class} = classbyname(shift) if scalar @_;
-	classbyval( $self->{class} || 1 ) if defined wantarray;
+	return $self->{class} = classbyname(shift) if scalar @_;
+	return defined $self->{class} ? classbyval( $self->{class} ) : 'IN';
 }
 
 
@@ -396,20 +469,19 @@ Resource record time to live in seconds.
 # published API.  These are required for parsing BIND zone files but
 # should not be used in other contexts.
 my %unit = ( W => 604800, D => 86400, H => 3600, M => 60, S => 1 );
-%unit = ( %unit, map { /\D/ ? lc($_) : $_ } %unit );
 
 sub ttl {
 	my ( $self, $time ) = @_;
 
 	return $self->{ttl} || 0 unless defined $time;		# avoid defining rr->{ttl}
 
-	my $ttl = 0;
+	my $ttl	 = 0;
 	my %time = reverse split /(\D)\D*/, $time . 'S';
 	while ( my ( $u, $t ) = each %time ) {
-		my $scale = $unit{$u} || die qq(bad time: $t$u);
+		my $scale = $unit{uc $u} || die qq(bad time: $t$u);
 		$ttl += $t * $scale;
 	}
-	$self->{ttl} = $ttl;
+	return $self->{ttl} = $ttl;
 }
 
 
@@ -421,31 +493,31 @@ sub ttl {
 
 sub _decode_rdata {			## decode rdata from wire-format octet string
 	my ( $self, $data, $offset ) = @_;
-	$self->{rdata} = substr $$data, $offset, $self->{rdlength};
+	return $self->{rdata} = substr $$data, $offset, $self->{rdlength};
 }
 
 
 sub _encode_rdata {			## encode rdata as wire-format octet string
-	my $rdata = shift->{rdata};
-	return defined $rdata ? $rdata : '';
+	return shift->{rdata};
 }
 
 
 sub _format_rdata {			## format rdata portion of RR string
-	my $self = shift;
-	my $data = $self->_encode_rdata;
-	my $size = length($data) || return '';
-	join ' ', '\\#', $size, unpack 'H*', $data;		# RFC3597 unknown RR format
+	my $rdata = shift->rdata;				# RFC3597 unknown RR format
+	return ( '\\#', length($rdata), split /(\S{32})/, unpack 'H*', $rdata );
 }
 
 
 sub _parse_rdata {			## parse RR attributes in argument list
 	my $self = shift;
-	die join ' ', $self->type, 'not implemented' if ref($self) eq __PACKAGE__;
+	die join ' ', 'type', $self->type, 'not implemented' if ref($self) eq __PACKAGE__;
 	die join ' ', 'no zone file representation defined for', $self->type;
 }
 
 
+sub _post_parse { }			## parser post processing
+
+
 sub _defaults { }			## set attribute default values
 
 
@@ -453,17 +525,17 @@ sub dump {				## print internal data structure
 	require Data::Dumper;					# uncoverable pod
 	local $Data::Dumper::Maxdepth = $Data::Dumper::Maxdepth || 6;
 	local $Data::Dumper::Sortkeys = $Data::Dumper::Sortkeys || 1;
-	print Data::Dumper::Dumper(@_);
+	return print Data::Dumper::Dumper(@_);
 }
 
-sub rdatastr {				## obsolete RR subtype method
-	&rdstring;						# uncoverable pod
+sub rdatastr {				## historical RR subtype method
+	return &rdstring;					# uncoverable pod
 }
 
 
 =head2 rdata
 
-    $rr = new Net::DNS::RR( type => NULL, rdata => 'arbitrary' );
+    $rr = Net::DNS::RR->new( type => NULL, rdata => 'arbitrary' );
 
 Resource record data section when viewed as opaque octets.
 
@@ -472,13 +544,13 @@ Resource record data section when viewed as opaque octets.
 sub rdata {
 	my $self = shift;
 
-	return eval { $self->_encode_rdata( 0x4000, {} ); } unless scalar @_;
+	return $self->_empty ? '' : eval { $self->_encode_rdata( 0x4000, {} ) } unless @_;
 
-	my $rdata = shift;
-	my $rdlen = $self->{rdlength} = length $rdata;
-	my $hash  = {};
-	$self->_decode_rdata( \$rdata, 0, $hash ) if $rdlen;
-	croak 'found compression pointer in rdata' if keys %$hash;
+	my $data = shift || '';
+	my $hash = {};
+	$self->_decode_rdata( \$data, 0, $hash ) if ( $self->{rdlength} = length $data );
+	croak 'compression pointer in rdata'	 if keys %$hash;
+	return;
 }
 
 
@@ -492,28 +564,11 @@ Returns a string representation of the RR-specific data.
 
 sub rdstring {
 	my $self = shift;
-	my $coln = shift || 0;
-	my $cols = 80;
-
-	my @rdata = eval { $self->_format_rdata; };
 
+	my @rdata = $self->_empty ? () : eval { $self->_format_rdata };
 	carp $@ if $@;
 
-	my ( @line, @fill );
-	foreach (@rdata) {
-		if ( ( $coln += 1 + length ) > $cols ) {
-			push @line, join ' ', @fill;		# multi-line format
-			$coln = length;
-			@fill = ();
-		}
-		$coln = $cols if chomp;				# line terminator
-		push( @fill, $_ );
-	}
-	return join ' ', @fill unless scalar @line;		# simple RR
-
-	$line[0] = '( ' . $line[0];				# multi-line RR
-	return join "\n\t", @line, join ' ', @fill, ')' if $coln < $cols;
-	return join "\n\t", @line, join( ' ', @fill ), ')';
+	return join "\n\t", _wrap(@rdata);
 }
 
 
@@ -521,48 +576,12 @@ sub rdstring {
 
     $rdlength = $rr->rdlength;
 
-Returns the length of the encoded RR-specific data.
+Returns the uncompressed length of the encoded RR-specific data.
 
 =cut
 
 sub rdlength {
-	length shift->_encode_rdata;
-}
-
-
-=head2 plain
-
-    $plain = $rr->plain;
-
-Returns a simplified single line representation of the RR using the
-zone file format defined in RFC1035.  This facilitates interaction
-with programs like nsupdate which have simplified RR parsers.
-
-=cut
-
-sub plain {
-	join ' ', shift->token;
-}
-
-
-=head2 token
-
-    @token = $rr->token;
-
-Returns a token list representation of the RR zone file string.
-
-=cut
-
-sub token {
-	local $_ = shift->string;
-
-	# parse into quoted strings, contiguous non-whitespace and (discarded) comments
-	s/\\\\/\\092/g;						# disguise escaped escape
-	s/\\"/\\034/g;						# disguise escaped quote
-	s/\\\(/\\040/g;						# disguise escaped bracket
-	s/\\\)/\\041/g;						# disguise escaped bracket
-	s/\\;/\\059/g;						# disguise escaped semicolon
-	my @token = grep defined && length, split /$PARSE_REGEX/o;
+	return length shift->rdata;
 }
 
 
@@ -600,7 +619,7 @@ The above example is the sorting function implemented in MX.
 
 =cut
 
-use vars qw(%rrsortfunct);
+our %rrsortfunct;
 
 sub set_rrsort_func {
 	my $class     = shift;
@@ -609,6 +628,7 @@ sub set_rrsort_func {
 
 	my ($type) = $class =~ m/::([^:]+)$/;
 	$rrsortfunct{$type}{$attribute} = $function;
+	return;
 }
 
 
@@ -621,22 +641,22 @@ get_rrsort_func() returns a reference to the comparator function.
 
 =cut
 
-my $default = sub { $Net::DNS::a->canonical() cmp $Net::DNS::b->canonical(); };
+my $default = sub { return $Net::DNS::a->canonical() cmp $Net::DNS::b->canonical(); };
 
 sub get_rrsort_func {
-	my $class = shift;
+	my $class     = shift;
 	my $attribute = shift || 'default_sort';
 
 	my ($type) = $class =~ m/::([^:]+)$/;
 
-	$rrsortfunct{$type}{$attribute} || $default;
+	return $rrsortfunct{$type}{$attribute} || return $default;
 }
 
 
 ################################################################################
 #
-#  Net::DNS::RR->_subclass($rrtype)
-#  Net::DNS::RR->_subclass($rrtype, $default)
+#  Net::DNS::RR->_subclass($rrname)
+#  Net::DNS::RR->_subclass($rrname, $default)
 #
 # Create a new object blessed into appropriate RR subclass, after
 # loading the subclass module (if necessary).  A subclass with no
@@ -646,74 +666,132 @@ sub get_rrsort_func {
 # The optional second argument indicates that default values are
 # to be copied into the newly created object.
 
-use vars qw(%_LOADED %_MINIMAL);
-
-$_MINIMAL{ANY} = bless [type => 255], __PACKAGE__;
-%_LOADED = %_MINIMAL;
+our %_MINIMAL = ( 255 => bless ['type' => 255], __PACKAGE__ );
+our %_LOADED  = %_MINIMAL;
 
 sub _subclass {
-	my $class   = shift;
-	my $rrtype  = shift;
-	my $default = shift;
+	my ( $class, $rrname, $default ) = @_;
 
-	unless ( $_LOADED{$rrtype} ) {
-		my $number = typebyname($rrtype);
-		my $mnemon = typebyval($number);
+	unless ( $_LOADED{$rrname} ) {
+		my $rrtype = typebyname($rrname);
 
-		unless ( $_LOADED{$mnemon} ) {			# load once only
-			$mnemon =~ s/[^A-Za-z0-9]//g;		# expect the unexpected
+		unless ( $_LOADED{$rrtype} ) {			# load once only
+			local @INC = LIB;
 
-			my $module = join '::', __PACKAGE__, $mnemon;
-			my $subclass = eval("require $module") ? $module : __PACKAGE__;
+			my $identifier = typebyval($rrtype);
+			$identifier =~ s/\W/_/g;		# kosher Perl identifier
+
+			my $subclass = join '::', __PACKAGE__, $identifier;
+
+			unless ( eval "require $subclass" ) {	## no critic ProhibitStringyEval
+				push @INC, sub {
+					Net::DNS::Parameters::_typespec("$rrtype.RRTYPE");
+				};
+
+				$subclass = join '::', __PACKAGE__, "TYPE$rrtype";
+				eval "require $subclass";	## no critic ProhibitStringyEval
+			}
+
+			$subclass = __PACKAGE__ if $@;
 
 			# cache pre-built minimal and populated default object images
-			my @base = ( 'type' => $number, $@ ? ( 'exception' => $@ ) : () );
-
-			$_MINIMAL{$mnemon} = bless [@base], $subclass;
+			my @base = ( 'type' => $rrtype );
+			$_MINIMAL{$rrtype} = bless [@base], $subclass;
 
 			my $object = bless {@base}, $subclass;
 			$object->_defaults;
-			$_LOADED{$mnemon} = bless [%$object], $subclass;
+			$_LOADED{$rrtype} = bless [%$object], $subclass;
 		}
 
-		$_MINIMAL{$rrtype} = $_MINIMAL{$mnemon};
-		$_LOADED{$rrtype}  = $_LOADED{$mnemon};
+		$_MINIMAL{$rrname} = $_MINIMAL{$rrtype};
+		$_LOADED{$rrname}  = $_LOADED{$rrtype};
 	}
 
-	my $prebuilt = $default ? $_LOADED{$rrtype} : $_MINIMAL{$rrtype};
-	bless {@$prebuilt}, ref($prebuilt);			# create object
+	my $prebuilt = $default ? $_LOADED{$rrname} : $_MINIMAL{$rrname};
+	return bless {@$prebuilt}, ref($prebuilt);		# create object
+}
+
+
+sub _annotation {
+	my $self = shift;
+	$self->{annotation} = ["@_"] if scalar @_;
+	return wantarray ? @{$self->{annotation} || []} : ();
+}
+
+
+my $warned;
+
+sub _deprecate {
+	carp join ' ', 'deprecated method;', pop(@_) unless $warned++;
+	return;
+}
+
+
+my %ignore = map { ( $_ => 1 ) } @core, 'annotation', '#';
+
+sub _empty {
+	my $self = shift;
+	return not( $self->{'#'} ||= scalar grep { !$ignore{$_} } keys %$self );
+}
+
+
+sub _wrap {
+	my @text = @_;
+	my $cols = 80;
+	my $coln = 0;
+
+	my ( @line, @fill );
+	foreach (@text) {
+		s/\\034/\\"/g;					# unescape "
+		s/\\092/\\\\/g;					# unescape escape
+		$coln += ( length || next ) + 1;
+		if ( $coln > $cols ) {				# start new line
+			push( @line, join ' ', @fill ) if @fill;
+			$coln = length;
+			@fill = ();
+		}
+		$coln = $cols	  if chomp;			# force line break
+		push( @fill, $_ ) if length;
+	}
+	push @line, join ' ', @fill;
+	return @line;
 }
 
 
 ################################################################################
 
-use vars qw($AUTOLOAD);
+our $AUTOLOAD;
 
 sub DESTROY { }				## Avoid tickling AUTOLOAD (in cleanup)
 
 sub AUTOLOAD {				## Default method
-	my $self = shift;
-	my $oref = ref($self);
-	confess 'undefined method ', $AUTOLOAD unless $oref;
-	confess "unknown RRtype\n" . $self->{exception} if $oref eq __PACKAGE__;
-
-	no strict q/refs/;
+	my $self     = shift;
 	my ($method) = reverse split /::/, $AUTOLOAD;
+
+	for ($method) {			## tolerate mixed-case attribute name
+		return $self->$_(@_) if tr [A-Z-] [a-z_];
+	}
+
+	no strict 'refs';		## no critic ProhibitNoStrict
 	*{$AUTOLOAD} = sub {undef};	## suppress repetition and deep recursion
+	my $oref = ref($self);
+	croak qq[$self has no class method "$method"] unless $oref;
 
 	my $string = $self->string;
-	my @object = grep defined($_), $oref->VERSION, $oref;
+	my @object = grep { defined($_) } $oref, $oref->VERSION;
+	my $module = join '::', __PACKAGE__, $self->type;
+	eval("require $module") if $oref eq __PACKAGE__;	## no critic ProhibitStringyEval
 
-	@_ = (<<"END");
-***  FATAL PROGRAM ERROR!!	Unknown method '$method'
+	@_ = ( <<"END", $@, "@object" );
+***  FATAL PROGRAM ERROR!!	Unknown instance method "$method"
 ***  which the program has attempted to call for the object:
 ***
 $string
 ***
-***  The @object object has no method '$method'
 ***  THIS IS A BUG IN THE CALLING SOFTWARE, which incorrectly assumes
 ***  that the object would be of a particular type.  The type of an
 ***  object should be checked before calling any of its methods.
+***
 END
 	goto &{'Carp::confess'};
 }
