@@ -1,75 +1,66 @@
 package Net::DNS::RR::TSIG;
 
-#
-# $Id: TSIG.pm 1561 2017-04-19 13:08:13Z willem $
-#
-our $VERSION = (qw$LastChangedRevision: 1561 $)[1];
-
-
 use strict;
 use warnings;
+our $VERSION = (qw$Id: TSIG.pm 1814 2020-10-14 21:49:16Z willem $)[2];
+
 use base qw(Net::DNS::RR);
 
+
 =head1 NAME
 
 Net::DNS::RR::TSIG - DNS TSIG resource record
 
 =cut
 
-
 use integer;
 
 use Carp;
 
-eval 'require Digest::HMAC';
-eval 'require Digest::MD5';
-eval 'require Digest::SHA';
-eval 'require MIME::Base64';
-
 use Net::DNS::DomainName;
-use Net::DNS::Parameters;
+use Net::DNS::Parameters qw(:class :type :rcode);
 
-use constant ANY  => classbyname qw(ANY);
-use constant TSIG => typebyname qw(TSIG);
+use constant SYMLINK => defined(&CORE::readlink);		# Except Win32, VMS, RISC OS
+
+use constant ANY  => classbyname q(ANY);
+use constant TSIG => typebyname q(TSIG);
+
+eval { require Digest::HMAC };
+eval { require Digest::MD5 };
+eval { require Digest::SHA };
+eval { require MIME::Base64 };
 
 {
 	# source: http://www.iana.org/assignments/tsig-algorithm-names
 	my @algbyname = (
-		'HMAC-MD5.SIG-ALG.REG.INT' => 157,
-		'HMAC-SHA1'		   => 161,
+		'HMAC-MD5.SIG-ALG.REG.INT' => 157,		# numbers are from ISC BIND keygen
+		'HMAC-SHA1'		   => 161,		# and not blessed by IANA
 		'HMAC-SHA224'		   => 162,
 		'HMAC-SHA256'		   => 163,
 		'HMAC-SHA384'		   => 164,
 		'HMAC-SHA512'		   => 165,
 		);
 
-	my @algbyalias = (
+	my @algalias = (
 		'HMAC-MD5' => 157,
 		'HMAC-SHA' => 161,
 		);
 
-
 	my %algbyval = reverse @algbyname;
 
-	my $map = sub {
-		my $arg = shift;
-		return $arg if $arg =~ /^\d/;
-		$arg =~ s/[^A-Za-z0-9]//g;			# strip non-alphanumerics
-		uc($arg);
-	};
-
-	my @pairedval = sort ( 1 .. 254, 1 .. 254 );		# also accept number
-	my %algbyname = map &$map($_), @algbyalias, @algbyname, @pairedval;
+	my @algrehash = map { /^\d/ ? ($_) x 3 : uc($_) } @algbyname, @algalias;
+	foreach (@algrehash) { s/[\W_]//g; }			# strip non-alphanumerics
+	my %algbyname = @algrehash;				# work around broken cperl
 
 	sub _algbyname {
 		my $key = uc shift;				# synthetic key
-		$key =~ s/[^A-Z0-9]//g;				# strip non-alphanumerics
-		$algbyname{$key};
+		$key =~ s/[\W_]//g;				# strip non-alphanumerics
+		return $algbyname{$key};
 	}
 
 	sub _algbyval {
 		my $value = shift;
-		$algbyval{$value};
+		return $algbyval{$value};
 	}
 }
 
@@ -79,7 +70,7 @@ sub _decode_rdata {			## decode rdata from wire-format octet string
 	my ( $data, $offset ) = @_;
 
 	my $limit = $offset + $self->{rdlength};
-	( $self->{algorithm}, $offset ) = decode Net::DNS::DomainName(@_);
+	( $self->{algorithm}, $offset ) = Net::DNS::DomainName->decode(@_);
 
 	# Design decision: Use 32 bits, which will work until the end of time()!
 	@{$self}{qw(time_signed fudge)} = unpack "\@$offset xxN n", $$data;
@@ -99,6 +90,7 @@ sub _decode_rdata {			## decode rdata from wire-format octet string
 	croak('misplaced or corrupt TSIG') unless $limit == length $$data;
 	my $raw = substr $$data, 0, $self->{offset};
 	$self->{rawref} = \$raw;
+	return;
 }
 
 
@@ -138,13 +130,14 @@ sub _defaults {				## specify RR attribute default values
 	$self->error(0);
 	$self->fudge(300);
 	$self->other('');
+	return;
 }
 
 
 sub _size {				## estimate encoded size
-	my $self = shift;
-	my $clone = bless {%$self}, ref($self);			   # shallow clone
-	length $clone->encode( 0, undef, new Net::DNS::Packet() );
+	my $self  = shift;
+	my $clone = bless {%$self}, ref($self);			# shallow clone
+	return length $clone->encode( 0, undef, Net::DNS::Packet->new() );
 }
 
 
@@ -153,7 +146,7 @@ sub encode {				## overide RR method
 
 	my $kname = $self->{owner}->encode();			# uncompressed key name
 	my $rdata = eval { $self->_encode_rdata(@_) } || '';
-	pack 'a* n2 N n a*', $kname, TSIG, ANY, 0, length $rdata, $rdata;
+	return pack 'a* n2 N n a*', $kname, TSIG, ANY, 0, length $rdata, $rdata;
 }
 
 
@@ -181,25 +174,24 @@ QQ
 }
 
 
-sub algorithm { &_algorithm; }
+sub algorithm { return &_algorithm; }
 
 
 sub key {
 	my $self = shift;
-
-	$self->keybin( MIME::Base64::decode( join "", @_ ) ) if scalar @_;
-	MIME::Base64::encode( $self->keybin(), "" ) if defined wantarray;
+	return MIME::Base64::encode( $self->keybin(), "" ) unless scalar @_;
+	return $self->keybin( MIME::Base64::decode( join "", @_ ) );
 }
 
 
-sub keybin { &_keybin; }
+sub keybin { return &_keybin; }
 
 
 sub time_signed {
 	my $self = shift;
 
 	$self->{time_signed} = 0 + shift if scalar @_;
-	$self->{time_signed} = time() unless $self->{time_signed};
+	return $self->{time_signed} ? $self->{time_signed} : ( $self->{time_signed} = time() );
 }
 
 
@@ -207,15 +199,14 @@ sub fudge {
 	my $self = shift;
 
 	$self->{fudge} = 0 + shift if scalar @_;
-	$self->{fudge} || 0;
+	return $self->{fudge} || 0;
 }
 
 
 sub mac {
 	my $self = shift;
-
-	$self->macbin( pack "H*", map { die "!hex!" if m/[^0-9A-Fa-f]/; $_ } join "", @_ ) if scalar @_;
-	unpack "H*", $self->macbin() if defined wantarray;
+	return MIME::Base64::encode( $self->macbin(), "" ) unless scalar @_;
+	return $self->macbin( MIME::Base64::decode( join "", @_ ) );
 }
 
 
@@ -223,16 +214,14 @@ sub macbin {
 	my $self = shift;
 
 	$self->{macbin} = shift if scalar @_;
-	$self->{macbin} || "";
+	return $self->{macbin} || "";
 }
 
 
 sub prior_mac {
 	my $self = shift;
-	my @args = map { /[^0-9A-Fa-f]/ ? croak "corrupt hexadecimal" : $_ } @_;
-
-	$self->prior_macbin( pack "H*", join "", @args ) if scalar @args;
-	unpack "H*", $self->prior_macbin() if defined wantarray;
+	return MIME::Base64::encode( $self->prior_macbin(), "" ) unless scalar @_;
+	return $self->prior_macbin( MIME::Base64::decode( join "", @_ ) );
 }
 
 
@@ -240,16 +229,14 @@ sub prior_macbin {
 	my $self = shift;
 
 	$self->{prior_macbin} = shift if scalar @_;
-	$self->{prior_macbin} || "";
+	return $self->{prior_macbin} || "";
 }
 
 
 sub request_mac {
 	my $self = shift;
-	my @args = map { /[^0-9A-Fa-f]/ ? croak "corrupt hexadecimal" : $_ } @_;
-
-	$self->request_macbin( pack "H*", join "", @args ) if scalar @args;
-	unpack "H*", $self->request_macbin() if defined wantarray;
+	return MIME::Base64::encode( $self->request_macbin(), "" ) unless scalar @_;
+	return $self->request_macbin( MIME::Base64::decode( join "", @_ ) );
 }
 
 
@@ -257,7 +244,7 @@ sub request_macbin {
 	my $self = shift;
 
 	$self->{request_macbin} = shift if scalar @_;
-	$self->{request_macbin} || "";
+	return $self->{request_macbin} || "";
 }
 
 
@@ -265,14 +252,14 @@ sub original_id {
 	my $self = shift;
 
 	$self->{original_id} = 0 + shift if scalar @_;
-	$self->{original_id} || 0;
+	return $self->{original_id} || 0;
 }
 
 
 sub error {
 	my $self = shift;
 	$self->{error} = rcodebyname(shift) if scalar @_;
-	rcodebyval( $self->{error} );
+	return rcodebyval( $self->{error} );
 }
 
 
@@ -280,21 +267,21 @@ sub other {
 	my $self = shift;
 	$self->{other} = shift if scalar @_;
 	my $time = $self->{error} == 18 ? pack 'xxN', time() : '';
-	$self->{other} = $time unless $self->{other};
+	return $self->{other} ? $self->{other} : ( $self->{other} = $time );
 }
 
 
-sub other_data { &other; }					# uncoverable pod
+sub other_data { return &other; }				# uncoverable pod
 
 
 sub sig_function {
 	my $self = shift;
 
-	return $self->{sig_function} unless scalar @_;
-	$self->{sig_function} = shift;
+	$self->{sig_function} = shift if scalar @_;
+	return $self->{sig_function};
 }
 
-sub sign_func { &sig_function; }				# uncoverable pod
+sub sign_func { return &sig_function; }				# uncoverable pod
 
 
 sub sig_data {
@@ -302,10 +289,10 @@ sub sig_data {
 
 	if ( ref($message) ) {
 		die 'missing packet reference' unless $message->isa('Net::DNS::Packet');
-		my @unsigned = grep ref($_) ne ref($self), @{$message->{additional}};
+		my @unsigned = grep { ref($_) ne ref($self) } @{$message->{additional}};
 		local $message->{additional} = \@unsigned;	# remake header image
 		my @part = qw(question answer authority additional);
-		my @size = map scalar( @{$message->{$_}} ), @part;
+		my @size = map { scalar @{$message->{$_}} } @part;
 		if ( my $rawref = $self->{rawref} ) {
 			delete $self->{rawref};
 			my $hbin = pack 'n6', $self->original_id, $message->{status}, @size;
@@ -356,7 +343,7 @@ sub create {
 		if ( $karg->isa('Net::DNS::Packet') ) {
 			my $sigrr = $karg->sigrr;
 			croak 'no TSIG in request packet' unless defined $sigrr;
-			return new Net::DNS::RR(		# ( request, options )
+			return Net::DNS::RR->new(		# ( request, options )
 				name	       => $sigrr->name,
 				type	       => 'TSIG',
 				algorithm      => $sigrr->algorithm,
@@ -370,7 +357,7 @@ sub create {
 			return $tsig;
 
 		} elsif ( ref($karg) eq 'Net::DNS::RR::KEY' ) {
-			return new Net::DNS::RR(
+			return Net::DNS::RR->new(
 				name	  => $karg->name,
 				type	  => 'TSIG',
 				algorithm => $karg->algorithm,
@@ -379,48 +366,50 @@ sub create {
 				);
 		}
 
-		croak "Usage:	create $class(keyfile)\n\tcreate $class(keyname, key)"
+		croak "Usage:	$class->create( \$keyfile, \@options )";
 
 	} elsif ( scalar(@_) == 1 ) {
-		my $key = shift;				# ( keyname, key )
-		return new Net::DNS::RR(
+		$class->_deprecate('create( $keyname, $key )'); # ( keyname, key )
+		return Net::DNS::RR->new(
 			name => $karg,
 			type => 'TSIG',
-			key  => $key
+			key  => shift
 			);
 
-	} elsif ( $karg =~ /[+.0-9]+private$/ ) {		# ( keyfile, options )
-		require File::Spec;
+	} else {
+		require File::Spec;				# ( keyfile, options )
 		require Net::DNS::ZoneFile;
-		my $keyfile = new Net::DNS::ZoneFile($karg);
-		my ( $alg, $key, $junk );
-		while ( my $line = $keyfile->_getline ) {
-			for ($line) {
-				( $junk, $alg ) = split if /Algorithm:/;
-				( $junk, $key ) = split if /Key:/;
-			}
+		my ($keypath) = SYMLINK ? grep( {$_} readlink($karg), $karg ) : $karg;
+		my ( $vol, $dir, $name ) = File::Spec->splitpath($keypath);
+		$name =~ m/^K([^+]+)\+\d+\+(\d+)\./;		# BIND dnssec-keygen
+		my ( $keyname, $keytag ) = ( $1, $2 );
+
+		my $keyfile = Net::DNS::ZoneFile->new($karg);
+		my ( $algorithm, $secret, $x );
+		while ( $keyfile->_getline ) {
+			/^key "([^"]+)"/     and $keyname   = $1;    # BIND tsig key
+			/algorithm ([^;]+);/ and $algorithm = $1;
+			/secret "([^"]+)";/  and $secret    = $1;
+
+			/^Algorithm:/ and ( $x, $algorithm ) = split;	 # BIND dnssec private key
+			/^Key:/	      and ( $x, $secret )    = split;
+
+			next unless /\bIN\s+KEY\b/;		# BIND dnssec public key
+			my $keyrr = Net::DNS::RR->new($_);
+			carp "$karg  does not appear to be a BIND dnssec public key"
+					unless $keytag and ( $keytag == $keyrr->keytag );
+			return $class->create( $keyrr, @_ );
 		}
 
-		my ( $vol, $dir, $file ) = File::Spec->splitpath( $keyfile->name );
-		my $kname;
-		$kname = $1 if $file =~ /^K([^+]+)+.+private$/;
-		return new Net::DNS::RR(
-			name	  => $kname,
-			type	  => 'TSIG',
-			algorithm => $alg,
-			key	  => $key,
-			@_
-			);
+		foreach ( $keyname, $algorithm, $secret ) {
+			croak 'key file incompatible with TSIG' unless $_;
+		}
 
-	} else {						# ( keyfile, options )
-		require Net::DNS::ZoneFile;
-		my $keyrr = new Net::DNS::ZoneFile($karg)->read;
-		croak 'key file incompatible with TSIG' unless $keyrr->type eq 'KEY';
-		return new Net::DNS::RR(
-			name	  => $keyrr->name,
+		return Net::DNS::RR->new(
+			name	  => $keyname,
 			type	  => 'TSIG',
-			algorithm => $keyrr->algorithm,
-			key	  => $keyrr->key,
+			algorithm => $algorithm,
+			key	  => $secret,
 			@_
 			);
 	}
@@ -431,16 +420,11 @@ sub verify {
 	my $self = shift;
 	my $data = shift;
 
-	unless ( abs( time() - $self->time_signed ) < $self->fudge ) {
-		$self->error(18);				# bad time
-		return;
-	}
-
 	if ( scalar @_ ) {
 		my $arg = shift;
 
 		unless ( ref($arg) ) {
-			$self->error(16);			# bad sig (multi-packet)
+			$self->error(16);			# BADSIG (multi-packet)
 			return;
 		}
 
@@ -448,12 +432,12 @@ sub verify {
 		if ( $arg->isa('Net::DNS::Packet') ) {
 			my $request = $arg->sigrr;		# request TSIG
 			my $rqstkey = lc( join '+', $request->name, $request->algorithm );
-			$self->error(17) unless $signerkey eq $rqstkey;
+			$self->error(17) unless $signerkey eq $rqstkey;			     # BADKEY
 			$self->request_macbin( $request->macbin );
 
 		} elsif ( $arg->isa(__PACKAGE__) ) {
 			my $priorkey = lc( join '+', $arg->name, $arg->algorithm );
-			$self->error(17) unless $signerkey eq $priorkey;
+			$self->error(17) unless $signerkey eq $priorkey;		     # BADKEY
 			$self->prior_macbin( $arg->macbin );
 
 		} else {
@@ -469,8 +453,9 @@ sub verify {
 	my $macbin = $self->macbin;
 	my $maclen = length $macbin;
 	my $minlen = length($tsigmac) >> 1;			# per RFC4635, 3.1
-	$self->error(16) unless $macbin eq substr $tsigmac, 0, $maclen;
-	$self->error(1) if $maclen < $minlen or $maclen < 10 or $maclen > length $tsigmac;
+	$self->error(16) if $macbin ne substr $tsigmac, 0, $maclen;			     # BADSIG
+	$self->error(22) if $maclen < $minlen or $maclen < 10 or $maclen > length $tsigmac;  # BADTRUNC
+	$self->error(18) if abs( time() - $self->time_signed ) > $self->fudge;		     # BADTIME
 
 	return $self->{error} ? undef : $tsig;
 }
@@ -502,17 +487,17 @@ sub vrfyerrstr {
 		if ( my $algname = shift ) {
 
 			unless ( my $digtype = _algbyname($algname) ) {
-				$self->{algorithm} = new Net::DNS::DomainName($algname);
+				$self->{algorithm} = Net::DNS::DomainName->new($algname);
 
 			} else {
 				$algname = _algbyval($digtype);
-				$self->{algorithm} = new Net::DNS::DomainName($algname);
+				$self->{algorithm} = Net::DNS::DomainName->new($algname);
 
 				my ( $hash, @param ) = @{$digest{$digtype}};
 				my ( undef, @block ) = @param;
-				my $digest   = new $hash(@param);
+				my $digest   = $hash->new(@param);
 				my $function = sub {
-					my $hmac = new Digest::HMAC( shift, $digest, @block );
+					my $hmac = Digest::HMAC->new( shift, $digest, @block );
 					$hmac->add(shift);
 					return $hmac->digest;
 				};
@@ -524,20 +509,20 @@ sub vrfyerrstr {
 			}
 		}
 
-		return $self->{algorithm}->name if defined wantarray;
+		return defined wantarray ? $self->{algorithm}->name : undef;
 	}
 
 
 	sub _keybin {			## install key in key table
 		my $self = shift;
 		croak 'Unauthorised access to TSIG key material denied' unless scalar @_;
-		my $keyref = $keytable{$self->{owner}->canonical} ||= {};
-		my $private = shift;	# closure keeps private key private
+		my $keyref  = $keytable{$self->{owner}->canonical} ||= {};
+		my $private = shift;				# closure keeps private key private
 		$keyref->{key} = sub {
 			my $function = $keyref->{digest};
 			return &$function( $private, @_ );
 		};
-		return undef;
+		return;
 	}
 
 
@@ -549,7 +534,7 @@ sub vrfyerrstr {
 		my $keyref = $keytable{$owner};
 		$keyref->{digest} = $self->sig_function unless $keyref->{digest};
 		my $function = $keyref->{key};
-		&$function(@_);
+		return &$function(@_);
 	}
 }
 
@@ -560,7 +545,7 @@ sub vrfyerrstr {
 sub _chain {
 	my $self = shift;
 	$self->{link} = undef;
-	bless {%$self, link => $self}, ref($self);
+	return bless {%$self, link => $self}, ref($self);
 }
 
 
@@ -571,9 +556,9 @@ __END__
 =head1 SYNOPSIS
 
     use Net::DNS;
-    $tsig = create Net::DNS::RR::TSIG( $keyfile );
+    $tsig = Net::DNS::RR::TSIG->create( $keyfile );
 
-    $tsig = create Net::DNS::RR::TSIG( $keyfile,
+    $tsig = Net::DNS::RR::TSIG->create( $keyfile,
 					fudge => 300
 					);
 
@@ -628,15 +613,12 @@ The default fudge is 300 seconds.
 
 =head2 mac
 
-    $mac = $rr->mac;
+    $rr->mac( $mac );
 
-Returns the message authentication code (MAC) as a string of hex
-characters.  The programmer must call the Net::DNS::Packet data()
+Message authentication code (MAC).
+The programmer must call the Net::DNS::Packet data()
 object method before this will return anything meaningful.
 
-=cut
-
-
 =head2 macbin
 
     $macbin = $rr->macbin;
@@ -686,7 +668,7 @@ The message ID from the header of the original packet.
      $rcode = $tsig->error;
 
 Returns the RCODE covering TSIG processing.  Common values are
-NOERROR, BADSIG, BADKEY, and BADTIME.  See RFC 2845 for details.
+NOERROR, BADSIG, BADKEY, and BADTIME.  See RFC2845-bis for details.
 
 
 =head2 other
@@ -702,7 +684,7 @@ case it will contain the server time as the number of seconds since
     sub signing_function {
 	my ( $keybin, $data ) = @_;
 
-	my $hmac = new Digest::HMAC( $keybin, 'Digest::MD5' );
+	my $hmac = Digest::HMAC->new( $keybin, 'Digest::MD5' );
 	$hmac->add( $data );
 	return $hmac->digest;
     }
@@ -717,25 +699,21 @@ The default signing function is HMAC-MD5.
 
      $sigdata = $tsig->sig_data($packet);
 
-Returns the packet packed according to RFC2845 in a form for signing. This
+Returns the packet packed according to RFC2845-bis in a form for signing. This
 is only needed if you want to supply an external signing function, such as is
 needed for TSIG-GSS.
 
 
 =head2 create
 
-    $tsig = create Net::DNS::RR::TSIG( $keyfile );
+    $tsig = Net::DNS::RR::TSIG->create( $keyfile );
 
-    $tsig = create Net::DNS::RR::TSIG( $keyfile,
+    $tsig = Net::DNS::RR::TSIG->create( $keyfile,
 					fudge => 300
 					);
 
 Returns a TSIG RR constructed using the parameters in the specified
-key file, which is assumed to have been generated by dnssec-keygen.
-
-    $tsig = create Net::DNS::RR::TSIG( $keyname, $key );
-
-The two argument form is supported for backward compatibility.
+key file, which is assumed to have been generated by tsig-keygen.
 
 =head2 verify
 
@@ -752,43 +730,34 @@ packet data conforms to the data in the TSIG itself
 
 =head1 TSIG Keys
 
-TSIG keys are symmetric keys generated using dnssec-keygen:
+The TSIG authentication mechanism employs shared secret keys
+to establish a trust relationship between two entities.
 
-	$ dnssec-keygen -a HMAC-SHA1 -b 160 -n HOST <keyname>
+It should be noted that it is possible for more than one key
+to be in use simultaneously between any such pair of entities.
 
-	The key will be stored as a private and public keyfile pair
-	K<keyname>+161+<keyid>.private and K<keyname>+161+<keyid>.key
+TSIG keys are generated using the tsig-keygen utility
+distributed with ISC BIND:
 
-    where
-	<keyname> is the DNS name of the key.
+    tsig-keygen -a HMAC-SHA256 host1-host2.example.
 
-	<keyid> is the (generated) numerical identifier used to
-	distinguish this key.
+Other algorithms may be substituted for HMAC-SHA256 in the above example.
 
-Other algorithms may be substituted for HMAC-SHA1 in the above example.
-
-It is recommended that the keyname be globally unique and incorporate
-the fully qualified domain names of the resolver and nameserver in
-that order. It should be possible for more than one key to be in use
-simultaneously between any such pair of hosts.
-
-Although the formats differ, the private and public keys are identical
-and both should be stored and handled as secret data.
+These keys must be protected in a manner similar to private keys,
+lest a third party masquerade as one of the intended parties
+by forging the message authentication code (MAC).
 
 
 =head1 Configuring BIND Nameserver
 
-The following lines must be added to the /etc/named.conf file:
+The generated key must be added to the /etc/named.conf configuration
+or a separate file introduced by the $INCLUDE directive:
 
-    key <keyname> {
-	algorithm HMAC-SHA1;
-	secret "<keydata>";
+    key "host1-host2.example. {
+	algorithm hmac-sha256;
+	secret "Secret+known+only+by+participating+entities=";
     };
 
-<keyname> is the name of the key chosen when the key was generated.
-
-<keydata> is the key string extracted from the generated key file.
-
 
 =head1 ACKNOWLEDGMENT
 
@@ -815,7 +784,7 @@ Copyright (c)2000,2001 Michael Fuhr.
 
 Portions Copyright (c)2002,2003 Chris Reinhardt.
 
-Portions Copyright (c)2013 Dick Franks.
+Portions Copyright (c)2013,2020 Dick Franks.
 
 All rights reserved.
 
@@ -843,7 +812,7 @@ DEALINGS IN THE SOFTWARE.
 
 =head1 SEE ALSO
 
-L<perl>, L<Net::DNS>, L<Net::DNS::RR>, RFC2845, RFC4635
+L<perl>, L<Net::DNS>, L<Net::DNS::RR>, RFC2845-bis, RFC4635
 
 L<TSIG Algorithm Names|http://www.iana.org/assignments/tsig-algorithm-names>
 
